##############################################################################
#   Title: make_accountants.R
# Purpose: Combines data from internal databases to produce analysis data sets
#          for internal Stata analysis.
#   Owner: EZ
# Created: 2020-04-10
##############################################################################

init = function() {
  
  rdata <<- file.path(localcodedir, "rdata")
  
  noldatadir <<- file.path(james, "analysis/nol_takeup_levels/output")
  nolsubdir <<- file.path(james, "analysis/nol_takeup_submission/output")
  pubdir <<- file.path(james, "derived/public_firms/output")
  fpdir <<- file.path(james, "derived/firm_panel/output")
  ppdir <<- file.path(james, "raw/pop_panel_sql/output")
  nolddir <<- file.path(james, "derived/nol_data/output")
  prepdtadir <<- file.path(james, "derived/preparers_data/output")
  rawptindir <<- file.path(james, "raw/ptin/output")
  nolvadir <<- file.path(james, "derived/nol_value_added/output")
  nol98dir <<- file.path(james, "analysis/nol_npv_1998-2002/output")
  nolcovdir <<- file.path(james, "analysis/nol_takeup_cov/output")
  
  james_a <<- "/PATH/TO/JAMES"
  r_d <<- file.path(james, "raw")
  d_d <<- file.path(james, "derived")
  a_d <<- file.path(james, "analysis")
  r_ad <<- file.path(james_a, "raw")
  d_ad <<- file.path(james_a, "derived")
  a_ad <<- file.path(james_a, "analysis")
  
  # Configure environment.
  MAINROOT <<- path.expand(".")
  DATADIR_I <<- file.path(MAINROOT, "code/accountants/out/20191031")
  DATADIR_II <<- file.path(MAINROOT, "code/accountants/out/20160306")
  
  
}

# Figure 1
build_loss_history = function() {
  
  db = dbConnect(SQLite(), dbname=file.path(r_ad, "pop_panel_sql/output/pop_panel.db"))
  data = dbGetQuery(db , "select 
									S.ein, 
									S.nol_yr, 
									S.nom_refund_cb / C.cpi_13
										as sim_refund_13,
									S.nom_refund_obs / C.cpi_13
										as obs_refund_13,
									S.mean_revenue_13,
									S.mean_payroll_13
								from sample.baseline_sample as S
								join cpi.cpi as C
									on S.nol_yr = C.year;")
  carrybacks = data.table(data)	
  
  carrybacks$tax_yr = carrybacks$nol_yr
  
  output = carrybacks[, list(
    eligibles=.N,
    eligibles_big=sum(sim_refund_13 > 1e3),
    claimers=sum(obs_refund_13 > 0),
    claimers_big=sum(obs_refund_13 > 0 & sim_refund_13 > 1e3),
    sim_refunds=sum(as.numeric(sim_refund_13), na.rm=T), 
    actual_refunds=sum(as.numeric(obs_refund_13), na.rm=T)
  ), by=tax_yr]
  
  # Save CSV	
  write.csv(output, file = file.path(DATADIR_II, "loss_history.csv"))
}

# Figure 3
build_mistake = function() {
  
  # npv_sim
  db = dbConnect(SQLite(), dbname=file.path(a_d, "nol_npv_1998-2002/output/data.db"))
  dbGetQuery(db, gsub("%s", a_d, "attach '%s/nol_takeup_submission/output/data.db' as data"))
  
  npv_sim = dbGetQuery(db, "select C.*, S.size_cat 
								from npv_sim_ub C
								left join data.size_cat as S
									on C.ein = S.ein
									and C.nol_yr = S.tax_yr;")
  npv_sim = data.table(npv_sim, key="ein")
  load(file.path(rdata, "carrybacks.RData"))
  size = carrybacks[, list(
    mean_payroll_13=mean(mean_payroll_13), 
    mean_revenue_13=mean(mean_revenue_13)), keyby=ein]
  npv_sim = size[npv_sim]	
  db = dbConnect(SQLite(), dbname=file.path(a_d, "nol_takeup_levels/output/data.db"))
  dbGetQuery(db, gsub("%s", a_d, "attach '%s/nol_takeup_submission/output/data.db' as data"))
  
  # clients
  clients = dbGetQuery(db, "select C.*, S.size_cat 
								from clients_1998_2011 C
								join data.size_cat as S
								    on C.ein = S.ein
									and C.nol_yr = S.tax_yr;")
  
  clients = data.table(clients, key="ein")
  load(file.path(rdata, "carrybacks.RData"))
  size = carrybacks[, list(
    mean_payroll_13=mean(mean_payroll_13), 
    mean_revenue_13=mean(mean_revenue_13)), keyby=ein]
  clients = size[clients]
  
  
  # Make intelligent size groups (consistent across the data)
  d_all = clients
  d_all$size_group <- cut(d_all$mean_revenue_13, 
                          quantile(d_all$mean_revenue_13,  
                                   c(seq(0,.9,.1),seq(.92,1,.02)), na.rm=T),
                          include.lowest=T)
  d_all = d_all[!is.na(size_group)]
  d_all$refund_group <- cut(d_all$sim_refund_13,
                            quantile(d_all$sim_refund_13, 
                                     c(seq(0,.9,.1), seq(.91,1,.01)), na.rm=T),
                            include.lowest=T)
  
  d_npv = npv_sim
  d_npv$size_group <- cut(d_npv$mean_revenue_13,
                          quantile(d_all$mean_revenue_13,  
                                   c(seq(0,.9,.1),seq(.92,1,.02)), na.rm=T),
                          include.lowest=T)
  
  d_npv$mistake <- with(d_npv, Iobs_refund == 0 & npv_cb_7r-npv_cf_7r > 0)
  d_npv$true_eligible <- with(d_npv, as.integer(npv_cb_7r-npv_cf_7r > 0))
  
  # Stats for all
  stats_all = cbind(
    d_all[, list(unclaimed=mean(Iobs_refund == 0), .N), by=size_group][order(size_group)],
    d_all[sim_refund_13 > 1e4, list(unclaimed4=mean(Iobs_refund == 0), N4=.N), by=size_group][order(size_group)],
    d_all[sim_refund_13 > 1e5, list(unclaimed5=mean(Iobs_refund == 0), N5=.N), by=size_group][order(size_group)]
  )
  
  stats_refund_all = d_all[, list(unclaimed=mean(Iobs_refund == 0), .N), by=refund_group][order(refund_group)]
  
  # Stats for NPV
  stats_npv = cbind(
    d_npv[, list(unclaimed=mean(Iobs_refund == 0), 
                 mistake=mean(mistake == 1), .N), by=size_group][order(size_group)],
    d_npv[npv_cb_7r > 1e4, 
          list(unclaimed_4=mean(Iobs_refund == 0), 
               mistake4=mean(mistake == 1), N4=.N), by=size_group][order(size_group)],
    d_npv[npv_cb_7r > 1e5, 
          list(unclaimed_5=mean(Iobs_refund == 0), 
               mistake5=mean(mistake == 1), N5=.N), by=size_group][order(size_group)]
  )
  
  write.csv(stats_all, file=file.path(DATADIR_II, "size_claim_all.csv"))
  write.csv(stats_refund_all, file=file.path(DATADIR_II, "refund_claim_all.csv"))
  write.csv(stats_npv, file=file.path(DATADIR_II, "size_claim_npv.csv"))
}

# Figure 4
build_preparer_covars = function() {
  
  db = dbConnect(SQLite(), dbname=file.path(a_ad, "nol_takeup_cov/output/data.db"))
  data = data.table(dbGetQuery(db, "select * from covariance;"))
  
  data$ub_same = data$cov_same + data$se_same*1.96
  data$lb_same = data$cov_same - data$se_same*1.96
  data$ub_diff = data$cov_diff + data$se_diff*1.96
  data$lb_diff = data$cov_diff - data$se_diff*1.96
  data$ub_mu = data$cov_mu + data$se_mu*1.96
  data$lb_mu = data$cov_mu - data$se_mu*1.96
  
  # Save CSV	
  write.csv(data, file=file.path(DATADIR_II, "cov.csv"))
  
}

################################################################################ 
# Main
################################################################################ 
main <- function() {
  
  init()
  
  build_loss_history()
  build_mistake()
  build_preparer_covars()

}

main
